package org.implab.gradle.containers.tasks;

import java.io.File;
import java.util.Optional;

import org.gradle.api.DefaultTask;
import org.gradle.api.file.Directory;
import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.logging.Logger;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.implab.gradle.containers.ContainerExtension;
import org.implab.gradle.containers.PropertiesMixin;
import org.implab.gradle.containers.cli.DockerTraits;

public abstract class DockerCliTask extends DefaultTask implements PropertiesMixin {

    @Input
    public abstract Property<String> getCliCmd();

    @Input
    @org.gradle.api.tasks.Optional
    public abstract DirectoryProperty getWorkingDirectory();

    @Internal
    protected ContainerExtension getContainerExtension() {
        return getProject().getExtensions().getByType(ContainerExtension.class);
    }

    public DockerCliTask() {
        getCliCmd().convention(getContainerExtension().getCliCmd());

    }
    
    protected DockerTraits docker() {
        return new TaskDockerTraits();
    }

    class TaskDockerTraits extends DockerTraits {

        @Override
        public Logger getLogger() {
            return DockerCliTask.this.getLogger();
        }

        @Override
        public Optional<File> getWorkingDir() {
            return getWorkingDirectory()
            .map(Directory::getAsFile)
            .map(Optional::of)
            .getOrElse(Optional.empty());
        }

        @Override
        public String getCliCmd() {
            return DockerCliTask.this.getCliCmd().get();
        }

    }
}
